package codemaptournament;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.pojava.datetime.CalendarUnit;

import org.pojava.datetime.DateTime;

/**
 *
 * @author Paul
 */
public class AutoWeeklyMissionFetcher {

    public static final int MAX_MISSIONS = 5;
    public static final URL urlCodeMapBoard;
    public static final URL urlAdvancedSearch;
    public static boolean useAdvancedSearch = false;

    static {
        try {
            urlCodeMapBoard = new URL("http://knucklecracker.com/forums/index.php?board=16.0");
            urlAdvancedSearch = new URL("http://knucklecracker.com/forums/index.php?action=search2");
        } catch (MalformedURLException ex) {
            throw new RuntimeException(ex);
        }
    }
    static final String searchParams = "advanced=1&search=weekly&searchtype=1&userspec=*&minage=0&maxage=9999&subject_only=1&sort=ID_MSG%7Cdesc&brd%5B16%5D=16&submit=Search";
    // Find links in HTML (pick out the link to the thread page from the thread title)
    static final Pattern pLink = Pattern.compile("<a[^>]*href=\"([^\"]+)\"[^>]*>(.*?)</a>", Pattern.DOTALL);
    // From a thread title, match and extract the name of the tournament and its closing date
    // Must have Weekly 5 Map Tournament|Challenge, then the tourney name, then Closing or Ending [on], then the date (the remainder of the title)
    // Fields separated by dash-like characters \p{Pd} or spaces \s
    // see http://stackoverflow.com/a/6052271/1816736
    static final Pattern pWeeklyTournament = Pattern.compile("\\bWeekly\\s+(?:5|Five).*Map\\s+(?:Tournament|Challenge)[\\s\\p{Pd}]+(.*?)[\\s\\p{Pd}]+(?:Clos\\w*|End\\w*)\\s*(?:on\\s*)?\\b(.*?)\\s*$", Pattern.CASE_INSENSITIVE);
    // In a thread, extract the individual posts
    static final Pattern pPost = Pattern.compile("<div[^>]*class=\"post\"[^>]*>(.*?)</div>", Pattern.DOTALL);

    
    /**
     * Get a list of the forum tournaments, ordered by closing date.
     * The info for each week's tournament is loaded, but the 5 maps are not
     * all loaded. Only the first initialLoadCount are.
     * @param initialLoadCount how many tournaments to read maps for initially, or -1 for all
     * @return
     * @throws IOException if getWeeklyMissionList fails due to IO
     */
    public static List<ForumTournament> getForumTournaments(int initialLoadCount) throws IOException {
        List<WeekInfo> weekInfos = getWeeklyMissionList();
        Collections.sort(weekInfos); // Sort by closing time (most recent first)
        List<ForumTournament> forumTournaments = new ArrayList<ForumTournament>();
        int index = 0;
        for (WeekInfo weekInfo : weekInfos) {
            boolean loadInitially = initialLoadCount == -1 || index < initialLoadCount;
            forumTournaments.add(new ForumTournament(weekInfo, loadInitially));
            index++;
        }
        return forumTournaments;
    }

    /**
     * From a tournament's url get the first post, then get a list of missions in a table
     * @param weekInfo
     * @return (possibly empty) list of missions
     */
    public static List<Mission> getMissionsForWeek(WeekInfo weekInfo) throws IOException {
        List<Mission> missions = new ArrayList<Mission>();
        String thread = NetUtil.unescape(NetUtil.readCompletely(weekInfo.link));
        Matcher mPost = pPost.matcher(thread);
        try {
            // Look at each post (on the first page) in turn...
            while (mPost.find()) {
                String postContents = mPost.group(1);
                String[][] table = ClipboardUtil.extractTableFromHTMLString(postContents);
                if (table == null) continue; // No table in this post, try the next post.

                // Look at the contents of the table, try to extract mission information
                for (String[] row : table) {
                    if (row.length < 0) {
                        continue;
                    }
                    // skip any header row (assume it the first header is map name ...)
                    if (row[0].toLowerCase().startsWith("map name")) {
                        continue;
                    }
                    // Parse all subsequent rows of data as map name first, then data
                    final String mapName = row[0];
    //                System.out.println("got " + mapName);
                    final int otherStuff = row.length - 1;
                    int size = Mission.DEFAULT_SIZE;
                    int complexity = Mission.DEFAULT_COMPLEXITY;
                    if (otherStuff >= 2) {
                        // assume next two are size and complexity
                        String sSize = row[1];
                        String sComp = row[2];
                        size = Mission.getSizeFromString(sSize);
                        complexity = Mission.getComplexityFromString(sComp);
                    }
                    Mission m = new Mission(mapName, size, complexity);
                    missions.add(m);
                    if (missions.size() >= MAX_MISSIONS) {
                        break;
                    }
                }

                // No problems so far, accept the data from the table in this post
                break;
            }
        } catch (NullPointerException npe) {
            // usually due to table being null
            Logger.getLogger(AutoWeeklyMissionFetcher.class.getName()).log(Level.WARNING, "no table found for {0}", weekInfo.link);
        }
        return missions;
    }

    /**
     * Check the Code Map Discussion board for weekly tournament threads
     * @return weekly tournament information
     */
    public static List<WeekInfo> getWeeklyMissionList() throws IOException {
        List<WeekInfo> infos = new ArrayList<WeekInfo>();
        String html;
        if (useAdvancedSearch) html = NetUtil.readCompletelyPOST(urlAdvancedSearch, searchParams, NetUtil.KC_ENCODING);
        else                   html = NetUtil.readCompletely(urlCodeMapBoard);
        String codeBoard = NetUtil.unescape(html);
        Matcher mLinks = pLink.matcher(codeBoard);
        while (mLinks.find()) {
            String link = mLinks.group(1);
            String text = mLinks.group(2);
            Matcher mWeeklyTournament = pWeeklyTournament.matcher(text);
            if (mWeeklyTournament.find()) { // find not matches - for now accept&ignore fluff before and after
                String topic = mWeeklyTournament.group(1);
                String closedate_str = mWeeklyTournament.group(2);
//                System.out.println("Topic " + topic + " Closedate " + closedate_str);
                // parse the close date string (note: current year is assumed if unspecified)
                // TODO: maybe fix couple of bad parsings (maybe use thread's post date?)
/*
 * Topic Famous Addresses
   Parsed 4/5th July as 2005-07-04 00:00:00 cutoff= 2005-07-04 00:00:00
   -- badly formatted date, should guess next Sunday after post
 * Topic Blake's Poems
   Parsed 24 June as 2013-06-24 00:00:00
   -- no year present, should use year of post
 */


                DateTime closedate = new DateTime(closedate_str);
                final int SUNDAY = 1;
                // the cutoff date is midnight (0:00) on the Monday of the tournament
                // but sometimes the closing date is listed as the Sunday
                DateTime cutoff = closedate;
                if (closedate.weekday() == SUNDAY) {
                    // make the cutoff be at midnight on Monday, not midnight on Sunday
                    cutoff = closedate.shift(CalendarUnit.DAY, 1);
                }

                // note that local (midnight in forum time) is not always midnight in local time
//                    System.out.println("Parsed " + closedate_str + " as " + closedate + " cutoff= " + cutoff);
                WeekInfo weekInfo = new WeekInfo(text, topic, cutoff.toDate(), closedate_str, new URL(link));
                infos.add(weekInfo);
            }
        }
        return infos;
    }
}

