package codemaptournament;

/**
 * Generic markup languages: BBCode,
 * @author Paul
 */
public interface MarkupRenderer {
    /** Make the given text bold */
    public String bold(String s);
    /** Make the given text italic */
    public String italic(String s);
    /** Make the text the given color */
    public String color(String s, String colorname);
    /** For rank = 0,1,2 return gold,silver,bronze victor colors */
    public String getVictorColor(int rank);
    /** Create a table */
    public String table(String[][] table, boolean firstRowHeader);
    /** format larger as heading */
    public String large(String s);
    /** format a table element right-justified */
    public String tableRight(String s);
    /** format a link */
    public String url(String link, String displayText);
    /** a break of rendered content to the next line */
    public String newline();
    /** description */
    public String getDescription();

    // See http://knucklecracker.com/forums/index.php?topic=10374.msg68222#msg68222 for discussion on colors
    public static final String COLOR_GOLD = "GoldenRod";
    public static final String COLOR_SILVER = "#A0A0A0";
    public static final String COLOR_BRONZE = "#8C7853"; // #CD7F32 = bronze for wiki, #8C7853 = TP CW2, "#B8860B" = UK CW1
    public static final String[] VICTOR_COLORS = {COLOR_GOLD, COLOR_SILVER, COLOR_BRONZE};

    public static final String COLOR_MAPNAME = "red";
    public static final String COLOR_SIZE = "blue";
    public static final String COLOR_COMPLEXITY = "green";

    public static final String COLOR_GRAVITYDIR = "brown";
    public static final String COLOR_BUILDDIR = "teal";
    public static final String COLOR_HASEXPERIMENTALS = "purple";

    /** whether to make map names have the same color as the header, COLOR_MAPNAME */
    public static final boolean COLOR_MAP_NAMES = true;

    /** whether to indent all BBCode tables to make the prettier to read */
    public static final boolean INDENT_BBCODE_TABLES = true;

    public static MarkupRenderer BBCode = new MarkupRenderer() {
        public String bold(String s) {
            return "[b]" + s + "[/b]";
        }

        public String color(String s, String colorname) {
            return "[color=" + colorname + "]" + s + "[/color]";
        }

        public String getVictorColor(int rank) {
            return rank >= 0 && rank < VICTOR_COLORS.length ? VICTOR_COLORS[rank] : "";
        }

        public String italic(String s) {
            return "[i]" + s + "[/i]";
        }

        /** render BBCode table. firstRowHeader currently does nothing special. */
        public String table(String[][] table, boolean firstRowHeader) {
            StringBuilder sb = new StringBuilder();
            final String indentStr = "    ";
            sb.append("[table]\n");
            int rowIdx = 0;
            for (String[] row : table) {
                if (rowIdx == 0) {
                    // TODO format the header fancily??
                }
                sb.append("[tr]");
                if (INDENT_BBCODE_TABLES) {
                    sb.append("\n");
                }
                for (String elem : row) {
                    if (INDENT_BBCODE_TABLES) {
                        sb.append(indentStr);
                    }
                    sb.append("[td]");
                    sb.append(elem);
                    sb.append("[/td]");
                    if (INDENT_BBCODE_TABLES) {
                        sb.append("\n");
                    }
                }
                sb.append("[/tr]\n");
                ++rowIdx;
            }
            sb.append("[/table]\n");
            return sb.toString();
        }

        public String large(String s) {
            final String size = "11pt";
            return "[size=" + size + "]" + s + "[/size]";
        }

        public String tableRight(String s) {
            return "[right]" + s + "[/right]";
        }

        public String url(String link, String displayText) {
            return "[url=" + link + "]" + displayText + "[/url]";
        }

        public String getDescription() {
            return "a forum post";
        }

        public String newline() {
            return "\n";
        }
    };

    public static MarkupRenderer WikiCode = new MarkupRenderer() {
        public String bold(String s) {
            return "**" + s + "**";
        }

        public String italic(String s) {
            return "//" + s + "//";
        }

        public String color(String s, String colorname) {
            return "<color " + colorname + ">" + s + "</color>";
        }

        public String getVictorColor(int rank) {
            return BBCode.getVictorColor(rank);
        }

        public String table(String[][] table, boolean firstRowHeader) {
            StringBuilder sb = new StringBuilder();
            int rowIdx = 0;
            for (String[] row : table) {
                char delim = (rowIdx == 0 && firstRowHeader) ? '^' : '|';
                sb.append(delim);
                for (String elem : row) {
                    sb.append(' ');
                    sb.append(elem);
                    sb.append(' ');
                    sb.append(delim);
                }
                sb.append("\n");
                ++rowIdx;
            }
            return sb.toString();
        }

        public String large(String s) {
            return "<span style=\"font-size:150%\">" + s + "</span>";
        }

        public String tableRight(String s) {
            return "  " + s;
        }

        public String url(String link, String displayText) {
            return "[[" + link + "|" + displayText + "]]";
        }

        public String getDescription() {
            return "the wiki";
        }

        public String newline() {
            return "\\\\\n"; // \\ followed by newline
        }
    };

}
