package codemaptournament;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.Scanner;
import org.apache.commons.lang3.StringEscapeUtils;

/**
 * network-themed utilites
 * @author Paul
 */
public class NetUtil {
    public static final long READ_DELAY_MS = 100; // this might be insignificant compared to processing time, but hey, I'll be nice.
    private static RateLimiter rateLimiter = new RateLimiter(READ_DELAY_MS);

    /** unescape all HTML entities back into their original text */
    public static String unescape(String text) {
        return StringEscapeUtils
                .unescapeHtml4(text)         // ampersand codes etd
                .replaceAll("\u00A0", " ");  // HACK assume all non-breaking spaces are spurious, turn into space
                                             // I was getting &nbsp;'s and \xA0's just from "helpful" web browser
                                             // since extra spaces in HTML are discarded, it marked a second space.
    }

    /**
     * Encode a URL parameter thingy.
     * hello there, world goes to hello+there%2C+world
     * THIS IS UNTESTED WITH THE SCORE SCRIPT!! (I presume I'd need it, maybe)
     * @param text
     * @return
     */
    public static String escapeURL(String text) {
        try {
            return URLEncoder.encode(text, "UTF-8");
        } catch (UnsupportedEncodingException ex) {
            throw new RuntimeException(ex);
        }
    }

    // Note on encodings:
    // Looks like all knucklecracker.com forum pages are served as ISO-8859-1 (Latin-1), both in HTTP header and meta.
    // BUT: in a couple threads, the en dash is presented as 0x96, which is the Windows 1252 code page
    // (similar: http://stackoverflow.com/a/7873916/1816736)
    // The problem is mainly visible in the titles of tournaments: Weekly 5-map tournament [-] Blah Blah Blah
    // With Latin-1, an en dash &#8211; is properly converted - example http://knucklecracker.com/forums/index.php?topic=12315.0
    // Example bad punctuation http://knucklecracker.com/forums/index.php?topic=12275.0
    // http://en.wikipedia.org/wiki/Windows-1252: It is very common to mislabel Windows-1252 text with
    // the charset label ISO-8859-1. HTML5 says documents advertised as ISO-8859-1 should actually be
    // parsed with the Windows-1252 encoding.

    public static final String KC_ENCODING = "cp1252";
    public static final String KC_WIKI_ENCODING = "utf-8";
    public static final String FAKE_USERAGENT = "Mozilla/5.0 (Windows NT 6.1)";

    public static String readCompletely(URL url) throws IOException {
        return readCompletely(url, KC_ENCODING);
    }

    public static String readCompletely(URL url, String encoding) throws IOException {
        return readCompletely(url, encoding, null);
    }

    public static String readCompletely(URL url, String encoding, String userAgent) throws IOException {
        rateLimiter.proceed(); // maybe pause so we don't swamp server
        URLConnection conn = url.openConnection();
        if (userAgent != null)
            conn.setRequestProperty("User-Agent", userAgent);
        InputStream stream = conn.getInputStream();
        
        final String text = readStream(stream, encoding);
        checkText(text, url);
        return text;
    }
    
    public static String readStream(InputStream inputStream, String encoding) throws IOException {
        Scanner sc = new Scanner(inputStream, encoding);
        sc.useDelimiter("\\Z"); // end of string
        return sc.next();
    }

    /**
     * Read a URL using HTTP POST.
     * @param url URL to fetch
     * @param postParams parameters, already param1=a&param2=b encoded
     * @param encoding charset of resulting document
     * @return text content of result
     * @throws IOException
     */
    public static String readCompletelyPOST(URL url, String postParams, String encoding) throws IOException {
        // http://stackoverflow.com/a/4206094/1816736
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setDoOutput(true);
        connection.setDoInput(true);
        connection.setInstanceFollowRedirects(false);
        connection.setRequestMethod("POST");
        connection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
        connection.setRequestProperty("charset", "utf-8");
        connection.setRequestProperty("Content-Length", "" + Integer.toString(postParams.getBytes().length));
        connection.setUseCaches(false);
        DataOutputStream wr = new DataOutputStream(connection.getOutputStream ());
        wr.writeBytes(postParams);
        wr.flush();
        wr.close();
        final String text = readStream(connection.getInputStream(), encoding);
        checkText(text, url);
        return text;
    }

    /** debugging sanity checks for properly decoded HTML stuff */
    private static void checkText(String text, URL url) {
        if (text.contains("\uFFFD")) { // check for bad encoding
            System.err.println("Text for " + url + " has an unknown character \\uFFFD (\uFFFD)!");
        }
        if (text.contains("\u0096")) {
            System.err.println("Text for " + url + " has a bad CP 1525 dash");
        }
    }
}
